use std::borrow::Cow;

/// Removes all whitespace from a string if any line breaks are present.
///
/// Example: "foo bar" => "foo bar" but "foo \n bar" => "foobar".
pub(crate) fn remove_whitespace_if_line_break<'s>(input: &'s str) -> Cow<'s, str> {
    let mut state = RemoveWhitespaceIfLineBreakState::Normal;
    for (offset, c) in input.char_indices() {
        match (&mut state, c) {
            (RemoveWhitespaceIfLineBreakState::Normal, '\n') => {
                let mut ret = String::with_capacity(input.len());
                ret.push_str(&input[..offset]);
                state = RemoveWhitespaceIfLineBreakState::HasLineBreak(ret);
            }
            (RemoveWhitespaceIfLineBreakState::Normal, ' ' | '\t') => {
                state = RemoveWhitespaceIfLineBreakState::HasWhitespace(offset);
            }
            (RemoveWhitespaceIfLineBreakState::Normal, _) => {}
            (RemoveWhitespaceIfLineBreakState::HasWhitespace(first_whitespace_offset), '\n') => {
                let mut ret = String::with_capacity(input.len());
                ret.push_str(&input[..*first_whitespace_offset]);
                for c in input[*first_whitespace_offset..offset].chars() {
                    if !c.is_ascii_whitespace() {
                        ret.push(c);
                    }
                }
                state = RemoveWhitespaceIfLineBreakState::HasLineBreak(ret);
            }
            (RemoveWhitespaceIfLineBreakState::HasWhitespace(_), _) => {}
            (RemoveWhitespaceIfLineBreakState::HasLineBreak(_), ' ' | '\t' | '\r' | '\n') => {}
            (RemoveWhitespaceIfLineBreakState::HasLineBreak(ret), _) => {
                ret.push(c);
            }
        }
    }
    match state {
        RemoveWhitespaceIfLineBreakState::Normal => Cow::Borrowed(input),
        RemoveWhitespaceIfLineBreakState::HasWhitespace(_) => Cow::Borrowed(input),
        RemoveWhitespaceIfLineBreakState::HasLineBreak(ret) => Cow::Owned(ret),
    }
}

enum RemoveWhitespaceIfLineBreakState {
    Normal,
    HasWhitespace(usize),
    HasLineBreak(String),
}

/// Removes all line breaks from a string
///
/// Example: "foo bar" => "foo bar" but "foo \n bar" => "foo  bar".
pub(crate) fn remove_line_break<'s>(input: &'s str) -> Cow<'s, str> {
    let mut state = RemoveLineBreakState::Normal;
    for (offset, c) in input.char_indices() {
        match (&mut state, c) {
            (RemoveLineBreakState::Normal, '\n') => {
                let mut ret = String::with_capacity(input.len());
                ret.push_str(&input[..offset]);
                state = RemoveLineBreakState::HasLineBreak(ret);
            }
            (RemoveLineBreakState::Normal, _) => {}
            (RemoveLineBreakState::HasLineBreak(_), '\n') => {}
            (RemoveLineBreakState::HasLineBreak(ret), _) => {
                ret.push(c);
            }
        }
    }
    match state {
        RemoveLineBreakState::Normal => Cow::Borrowed(input),
        RemoveLineBreakState::HasLineBreak(ret) => Cow::Owned(ret),
    }
}

enum RemoveLineBreakState {
    Normal,
    HasLineBreak(String),
}
